// livido-lib.c
// LiVES
// (c) G. Finch 2005 <salsaman@xs4all.nl>
// Released under the Lesser GNU Public License (LGPL) version 2.1
// see www.gnu.org for licensing details 

// livido utility library



//int livido_init_port (int port_type) {

//}
#include <string.h>

#ifndef __LIVIDO_H__
#include "plugins-livido.h"
#endif

static void livido_atom_free (livido_atom_t *atom) {
  switch (atom->atom_type) {
  case LIVIDO_ATOM_TYPE_VOIDPTR:
  case LIVIDO_ATOM_TYPE_INT8PTR:
  case LIVIDO_ATOM_TYPE_UINT8PTR:
  case LIVIDO_ATOM_TYPE_INT16PTR:
  case LIVIDO_ATOM_TYPE_UINT16PTR:
  case LIVIDO_ATOM_TYPE_INT32PTR:
  case LIVIDO_ATOM_TYPE_UINT32PTR:
    break;
  default :
    free (atom->value);
  }
  free (atom);
  return;
}


static livido_atom_t *livido_create_atom (void *value, int atom_type, size_t size) {
  livido_atom_t *atom=malloc(sizeof(livido_atom_t));
  atom->size=size;
  atom->atom_type=atom_type;
  switch (atom->atom_type) {
  case LIVIDO_ATOM_TYPE_VOIDPTR:
  case LIVIDO_ATOM_TYPE_INT8PTR:
  case LIVIDO_ATOM_TYPE_UINT8PTR:
  case LIVIDO_ATOM_TYPE_INT16PTR:
  case LIVIDO_ATOM_TYPE_UINT16PTR:
  case LIVIDO_ATOM_TYPE_INT32PTR:
  case LIVIDO_ATOM_TYPE_UINT32PTR:
    atom->value=value;
    break;
  default :
    atom->value=malloc(size);
    memcpy (atom->value,value,size);
  }
  return atom;
}



static void livido_storage_free(livido_storage_t *store) {
  int i;
  
  if (store==NULL) return;
  
  if (store->num_elements==1) {
    livido_atom_free(store->atom);
  }
  else {
    for (i=0;i<store->num_elements;i++) {
      livido_atom_free(store->array[i]);
    }
  }
  
  free (store);
}






void livido_properties_free (livido_port_t *port) {
  livido_property_t *prop,*prop_next;

  if (port==NULL) return;
  prop=port[0];

  while (prop!=NULL) {
    free((void *)prop->key);
    livido_storage_free(prop->data);
    prop_next=prop->next;
    free(prop);
    prop=prop_next;
  }
}




void livido_parameters_free (livido_instance_t *inst) {
  int i;

  for (i=0;i<inst->num_in_parameters;i++) {
    livido_properties_free(inst->in_parameters[i]);
  }

  inst->num_in_parameters=0;
  inst->in_parameters=NULL;

  for (i=0;i<inst->num_out_parameters;i++) {
    livido_properties_free(inst->out_parameters[i]);
  }

  inst->num_out_parameters=0;
  inst->out_parameters=NULL;

}

static int livido_set_value (livido_port_t *port, const char *key, int atom_type, int num_elems, void *value, size_t size) {
  // if a property with "key" exists we will change the atom in the storage

  // otherwise we add a new property

  // size is only used for single atoms (num_elems==1)

  livido_property_t *prop,*last_prop=NULL;
  livido_storage_t *store;
  int i;
  char **valuecharptrptr;

  if (port==NULL) return LIVIDO_ERROR_NULL_PORT;
  prop=port[0];

  while (prop!=NULL) {
    if (!strcmp(prop->key,key)) {
      if (prop->flags&LIVIDO_PROPERTY_FLAG_READONLY) return LIVIDO_ERROR_PROPERTY_READONLY;
      store=prop->data;
      livido_storage_free(store);
      store=malloc(sizeof(livido_storage_t));
      store->num_elements=num_elems;
      if (num_elems==1) store->atom=livido_create_atom(value,atom_type,size);
      else {
	store->array=malloc(num_elems*sizeof(livido_atom_t *));
	for (i=0;i<num_elems;i++) {
	  switch (atom_type) {
	  case LIVIDO_ATOM_TYPE_CHARPTR:
	    valuecharptrptr=(char **)value;
	    store->array[i]=livido_create_atom(valuecharptrptr[i],atom_type,strlen(valuecharptrptr[i]));
	    break;
	    // TODO - other atom_types
	  }
	}
      }
      prop->data=store;
      return LIVIDO_NO_ERROR;
    }
    last_prop=prop;
    prop=prop->next;
  }
  
  prop=malloc(sizeof(livido_property_t));
  prop->key=strdup(key);
  store=malloc(sizeof(livido_storage_t));
  store->num_elements=num_elems;
  if (num_elems==1) store->atom=livido_create_atom(value,atom_type,size);
  else {
    store->array=malloc(num_elems*sizeof(livido_atom_t *));
    for (i=0;i<num_elems;i++) {
      switch (atom_type) {
      case LIVIDO_ATOM_TYPE_CHARPTR:
	valuecharptrptr=(char **)value;
	store->array[i]=livido_create_atom(valuecharptrptr[i],atom_type,strlen(valuecharptrptr[i]));
	break;
	// TODO - other atom_types
      }
    }
  }
  prop->data=store;
  prop->flags=0;
  if (last_prop!=NULL) last_prop->next=prop;
  prop->next=NULL;






  return LIVIDO_NO_ERROR;

}


int livido_get_num_elements (livido_port_t *port, const char *key) {

  livido_property_t *prop=NULL;
  if (port==NULL) return 0;

  prop=port[0];

  while (prop!=NULL) {
    if (!strcmp(prop->key,key)) {
      return prop->data->num_elements;
    }
    prop=prop->next;
  }
  return 0;
}



livido_storage_t *livido_get_storage_for(livido_port_t *port, const char *key) {

  livido_property_t *prop=NULL;
  if (port==NULL) return NULL;

  prop=port[0];

  while (prop!=NULL) {
    if (!strcmp(prop->key,key)) {
      return prop->data;
    }
    prop=prop->next;
  }
  return NULL;
}




int livido_get_value (livido_port_t *port, const char *key, int atom_type, int idx, void *value) {
  // get SINGLE value - index starts at 1
  livido_storage_t *store;
  
  store=livido_get_storage_for(port,key);
  if (store->num_elements<idx) return LIVIDO_ERROR_WRONG_ELEMENTS;

  if (store->num_elements==1) {
    memcpy(value,store->atom->value,store->atom->size);
    if (atom_type==LIVIDO_ATOM_TYPE_CHARPTR) memset(value+store->atom->size,0,1);
  }
  else {
    memcpy(value,store->array[idx-1]->value,store->array[idx-1]->size);
    if (atom_type==LIVIDO_ATOM_TYPE_CHARPTR) memset(value+store->array[idx-1]->size,0,1);
  }

  return LIVIDO_NO_ERROR;
}


/////////////////////////////////////////////////////////////////
// public functions - property setters


// TODO - *** use livido_get_size()



int livido_set_int32_prop (livido_parameter_t *param, const char *key, int value) {
  // returns a LIVIDO_ERROR

  return livido_set_value (param,key,LIVIDO_ATOM_TYPE_INT32,1,&value,sizeof(int32_t));
}

int livido_set_boolean_prop (livido_parameter_t *param, const char *key, int value) {
  // returns a LIVIDO_ERROR

  return livido_set_value (param,key,LIVIDO_ATOM_TYPE_BOOL,1,&value,sizeof(int32_t));
}


int livido_set_string_prop (livido_parameter_t *param, const char *key, char *value) {
  // returns a LIVIDO_ERROR

  return livido_set_value (param,key,LIVIDO_ATOM_TYPE_CHARPTR,1,value,strlen(value));
}


int livido_set_string_array(livido_parameter_t *param, const char *key, int num_elems, char **list) {

  return livido_set_value(param,key,LIVIDO_ATOM_TYPE_CHARPTR,num_elems,list,0);
}


///////////////////////////////////////////////////////////////////
// public functions - utilities

int livido_get_atom_size(livido_port_t *port, const char *key, int i) {
  // return the atom size for an atom for port/property
  livido_storage_t *store;

  store=livido_get_storage_for(port,key);
  if (store==NULL) return 0;

  // TODO
  if (store->num_elements==1) return store->atom->size;
  return store->array[i-1]->size;
}


int livido_get_atom_type(livido_port_t *port, const char *key) {

  livido_storage_t *store=livido_get_storage_for(port,key);
  if (store==NULL) return LIVIDO_ERROR_NOSUCH_PROPERTY;

  if (store->num_elements==1) return store->atom->atom_type;
  return store->array[0]->atom_type;

}

/////////////////////////////////////////////////////////////////
// public functions - property getters


int livido_get_int32_prop (livido_parameter_t *param, const char *key, int32_t *value) {
  // returns a LIVIDO_ERROR
  return livido_get_value( param, key, LIVIDO_ATOM_TYPE_INT32, 1, value);
}

int livido_get_boolean_prop (livido_parameter_t *param, const char *key, int32_t *value) {
  // returns a LIVIDO_ERROR
  return livido_get_value( param, key, LIVIDO_ATOM_TYPE_BOOL, 1, value);
}

int livido_get_string_prop (livido_parameter_t *param, const char *key, char *value) {
  // returns a LIVIDO_ERROR
  return livido_get_value( param, key, LIVIDO_ATOM_TYPE_CHARPTR, 1, value);
}

int livido_get_string_array (livido_parameter_t *param, const char *key, char **value) {
  livido_storage_t *store=livido_get_storage_for(param,key);
  int error=LIVIDO_NO_ERROR,i;

  if (store->num_elements==1) {
    value[0]=(char *)malloc(livido_get_atom_size(param,key,1)+1);
    error=livido_get_value( param, key, LIVIDO_ATOM_TYPE_CHARPTR, 1, value[0]);
    if (error!=LIVIDO_NO_ERROR) return error;
    value[1]=NULL;
  }
  else {
    for (i=1;i<=store->num_elements;i++) {
      value[i-1]=(char *)malloc(livido_get_atom_size(param,key,i)+1);
      error=livido_get_value( param, key, LIVIDO_ATOM_TYPE_CHARPTR, i,value[i-1]);
      if (error!=LIVIDO_NO_ERROR) return error;
    }
    value[i-1]=NULL;
  }
  return error;
}


//////////////////////////////////////////////////////////////

static livido_port_t *livido_init_port(int port_type) {
  livido_port_t *port=malloc(sizeof(livido_port_t));
  port[0]=malloc(sizeof(livido_property_t));
  port[0]->key=strdup("type");
  port[0]->flags=0;
  port[0]->next=NULL;
  port[0]->data=NULL;
  livido_set_int32_prop(port,"type",port_type);
  return port;
}



///////////////////////////////////////////////////////////////////////
// public functions - port initialisers



livido_parameter_t *livido_init_switch_param(char *name, int val) {
  livido_parameter_t *param = livido_init_port(LIVIDO_PORT_SWITCH);

  livido_set_string_prop(param,"name",name);
  livido_set_boolean_prop(param,"value",val);
  
  return param;
}



livido_parameter_t *livido_init_int32_param(char *name, int val, int min, int max) {
  livido_parameter_t *param = livido_init_port(LIVIDO_PORT_INT32);

  livido_set_string_prop(param,"name",name);
  livido_set_int32_prop(param,"value",val);
  livido_set_int32_prop(param,"min",min);
  livido_set_int32_prop(param,"max",max);
  
  return param;
}


livido_parameter_t *livido_init_string_list_param(char *name, int value, int num_elems, char **list) {
  livido_parameter_t *param = livido_init_port(LIVIDO_PORT_STRING_LIST);

  livido_set_string_prop(param,"name",name);
  livido_set_string_array(param,"list_values",num_elems,list);
  livido_set_int32_prop(param,"value",value);

  return param;
}  




