/*
 * EffecTV - Realtime Digital Video Effector
 * Copyright (C) 2001 FUKUCHI Kentarou
 *
 * 1DTV - scans line by line and generates amazing still image.
 * Copyright (C) 2001 FUKUCHI Kentarou
 *
 */


////////////////////////////////////////////////

/* modified for Livido by G. Finch (salsaman)
   modifications (c) G. Finch */


#include "plugins-livido.h"
#include "livido-lib.c"

typedef unsigned int RGB32;


struct _sdata {
  int line;
  RGB32 *linebuf;
};

livido_setup_f livido_setup;

livido_init_f oned_init;
livido_deinit_f oned_deinit;
livido_process_f oned_process;

/////////////////////////////////

livido_palette_t in_palettes[]={LIVIDO_PALETTE_RGB32,LIVIDO_PALETTE_END};
livido_palette_t out_palettes[]={LIVIDO_PALETTE_RGB32,LIVIDO_PALETTE_END};

livido_channel_template_t in_channels={"In 1",0,in_palettes,0,0,0,NULL};
livido_channel_template_t out_channels={"Out 1",LIVIDO_CHANNEL_SAME_AS_SIZE|LIVIDO_CHANNEL_SAME_AS_PALETTE,out_palettes,1,0,0,NULL};

livido_instance_template_t fx={"onedTV","effectTV","1 dimensional television !",1,1.7,LIVIDO_PROPERTY_REALTIME,NULL,NULL,&in_channels,&out_channels,oned_init,oned_deinit,oned_process,NULL,NULL,NULL};




livido_instance_template_t *livido_setup(void) {
  return &fx;
}




int oned_init(livido_instance_t *inst) {
  struct _sdata *sdata;
  int map_w;
  
  if (inst==NULL) return LIVIDO_ERROR_NULL_INSTANCE;

  map_w=inst->in_channels[0].width;

  sdata=malloc(sizeof(struct _sdata));

  if(sdata == NULL ) {
    return LIVIDO_ERROR_MEMORY_ALLOCATION;
  }

  sdata->linebuf = malloc(map_w*sizeof(RGB32 *));
  if(sdata->linebuf == NULL ) {
    free(sdata);
    return LIVIDO_ERROR_MEMORY_ALLOCATION;
  }

  bzero(sdata->linebuf, map_w*sizeof(RGB32));
  sdata->line = 0;

  inst->internal=sdata;

  return LIVIDO_NO_ERROR;
}



int oned_deinit(livido_instance_t *inst) {
  struct _sdata *sdata;
  if (inst==NULL) return LIVIDO_ERROR_NULL_INSTANCE;
  sdata=inst->internal;
  if (sdata != NULL) {
    free(sdata->linebuf);
    free(sdata);
  }
  return LIVIDO_NO_ERROR;
}



static void blitline(RGB32 *src,RGB32 *dest, int video_width, struct _sdata *sdata)
{
  src += video_width * sdata->line;
  dest += video_width * sdata->line;
  memcpy(dest, src, sizeof(RGB32) * video_width);
}


int oned_process (livido_instance_t *inst, livido_frame_t *in, livido_frame_t *out) {
  RGB32 *src,*dest;
  struct _sdata *sdata;
  int width,height;
  register int i;

  if (inst==NULL) return LIVIDO_ERROR_NULL_INSTANCE;

  sdata=inst->internal;
  src=in->pixel_data[0];
  dest=out->pixel_data[0];

  width = inst->in_channels[0].width;
  height = inst->in_channels[0].height;

  blitline(src,dest,width,sdata);
  sdata->line++;
  if(sdata->line >= height)
    sdata->line = 0;

  dest += width * sdata->line;
  for(i=0; i<width; i++) {
    dest[i] = 0xff00;
  }
  return LIVIDO_NO_ERROR;
}
