/*
 * EffecTV - Realtime Digital Video Effector
 * Copyright (C) 2001 FUKUCHI Kentarou
 *
 * Rotozoom - Ported to effectv by Buddy Smith.
 * Original license below
 *
 */

/* Copyright (C) 2002 W.P. van Paassen - peter@paassen.tmfweb.nl, Byron Ellacott - bje@apnic.net

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* 07-14-02 Major speed optimizations done by WP (it's really fast now :-)). Uses an 8 bit indexed image*/
/* note that the code has not been fully optimized */


/* modified for Livido by G. Finch (salsaman)
   modifications (c) G. Finch */

#include "plugins-livido.h"
#include "livido-lib.c"
#include "math.h"

struct _sdata {
  int *roto;
  int *roto2;
  int path;
  int zpath;
};

//////////////////////////////////

livido_setup_f livido_setup;

livido_init_f rotozoom_init;
livido_deinit_f rotozoom_deinit;
livido_process_f rotozoom_process;

typedef unsigned int RGB32;

/////////////////////////////////

livido_palette_t in_palettes[]={LIVIDO_PALETTE_RGB32,LIVIDO_PALETTE_END};
livido_palette_t out_palettes[]={LIVIDO_PALETTE_RGB32,LIVIDO_PALETTE_END};

livido_channel_template_t in_channels={"In 1",0,in_palettes,0,0,0,NULL};
livido_channel_template_t out_channels={"Out 1",LIVIDO_CHANNEL_SAME_AS_SIZE|LIVIDO_CHANNEL_SAME_AS_PALETTE,out_palettes,1,0,0,NULL};

livido_instance_template_t fx={"rotozoom","effectTV","Rotate and zoom",1,1.7,LIVIDO_PROPERTY_REALTIME,NULL,NULL,&in_channels,&out_channels,rotozoom_init,rotozoom_deinit,rotozoom_process,NULL,NULL,NULL};




livido_instance_template_t *livido_setup(void) {
  return &fx;
}




void draw_tile(int stepx, int stepy, int zoom,RGB32 *texture,RGB32 *image, int video_width, int video_height)
{
  int x, y, xd, yd, a, b, sx, sy;
  register int i,j;
    
  int video_area=video_width*video_height;
  sx = sy = 0;
  xd = (stepx * zoom) >> 12;
  yd = (stepy * zoom) >> 12;
    
  /* Stepping across and down the screen, each screen row has a
   * starting coordinate in the texture: (sx, sy).  As each screen
   * row is traversed, the current texture coordinate (x, y) is
   * modified by (xd, yd), which are (sin(rot), cos(rot)) multiplied
   * by the current zoom factor.  For each vertical step, (xd, yd)
   * is rotated 90 degrees, to become (-yd, xd).
   *
   * More fun can be had by playing around with x, y, xd, and yd as
   * you move about the image.
   */

  for (j = 0; j < video_height; j++) {
    x = sx; y = sy;   
    for (i = 0; i < video_width; i++) {
      a = x >> 12 &255;
      b = y >> 12 &255;
      a=a*video_width/255;
      b=b*video_height/255;
      //      a*=video_width/64;
      //      b*=video_height/64;
      if (b*video_width+a<video_area) *image++ = texture[b*video_width + a];
      else *image++=0;
      x += xd; y += yd;
    }
    sx -= yd; sy += xd;
  }
}





int rotozoom_init(livido_instance_t *inst) {
  int i;
  struct _sdata *sdata;
  int map_h,map_w;

  if (inst==NULL) return LIVIDO_ERROR_NULL_INSTANCE;

  map_h=inst->in_channels[0].height;
  map_w=inst->in_channels[0].width;

  sdata=malloc(sizeof(struct _sdata));

  if(sdata == NULL ) {
    return LIVIDO_ERROR_MEMORY_ALLOCATION;
  }

  sdata->roto = malloc(map_h*map_w*sizeof(RGB32 *));
  if(sdata->roto == NULL ) {
    free(sdata);
    return LIVIDO_ERROR_MEMORY_ALLOCATION;
  }
  sdata->roto2 = malloc(map_h*map_w*sizeof(RGB32 *));
  if(sdata->roto2 == NULL ) {
    free(sdata->roto);
    free(sdata);
    return LIVIDO_ERROR_MEMORY_ALLOCATION;
  }
  sdata->path=sdata->zpath=0;

  inst->internal=sdata;

  for (i = 0; i < 256; i++) {
    float rad =  (float)i * 1.41176 * 0.0174532;
    float c = sin(rad);
    sdata->roto[i] = (c + 0.8) * 4096.0;
    sdata->roto2[i] = (2.0 * c) * 4096.0;
  }

  inst->num_in_parameters=2;
  inst->in_parameters=malloc(2*sizeof(livido_parameter_t));
  inst->in_parameters[0]=livido_init_int32_param("Zoom (squared)",1000000,1000000,32768*32768);
  inst->in_parameters[1]=livido_init_switch_param("Auto zoom",1);

  return LIVIDO_NO_ERROR;
}


int rotozoom_deinit(livido_instance_t *inst) {
  struct _sdata *sdata;
  if (inst==NULL) return LIVIDO_ERROR_NULL_INSTANCE;
  sdata=inst->internal;
  if (sdata != NULL) {
    free(sdata->roto);
    free(sdata->roto2);
    free(sdata);
  }
  return LIVIDO_NO_ERROR;
}


int rotozoom_process (livido_instance_t *inst, livido_frame_t *in, livido_frame_t *out) {
  RGB32 *src,*dst;
  struct _sdata *sdata;
  int width,height;
  int val,zoom;

  if (inst==NULL) return LIVIDO_ERROR_NULL_INSTANCE;

  sdata=inst->internal;
  src=in->pixel_data[0];
  dst=out->pixel_data[0];

  width = inst->in_channels[0].width;
  height = inst->in_channels[0].height;

  livido_get_boolean_prop(inst->in_parameters[1],"value",&val);

  if (val) {
    zoom=sdata->roto2[sdata->zpath];
    sdata->zpath =(sdata->zpath + 1) & 255;
  }
  else {
    livido_get_int32_prop(inst->in_parameters[0],"value",&zoom);
    zoom=(int)sqrt(zoom*1.);
  }

  draw_tile(sdata->roto[sdata->path], sdata->roto[(sdata->path + 128) & 0xFF], zoom,src,dst,width,height);
  sdata->path = (sdata->path - 1) & 255;

  return LIVIDO_NO_ERROR;
}




