// effects-livido.h
// LiVES
// (c) G. Finch 2005 <salsaman@xs4all.nl>
// Released under the GPL 2.0
// see file COPYING or www.gnu.org for licensing details 

typedef livido_instance_template_t livido_class_t;

void livido_init_all(void); // called once on startup
void livido_end(void); // called on shutdown

GList *livido_get_all_names (gboolean add_type);
int livido_getmodespk (void);
gint livido_get_numeffects(void);



// playback functions
gboolean livido_playback_gen_start(void);
void livido_generator_end (livido_instance_t *src);
void livido_bg_generator_end (livido_instance_t *src);


/////////////////////////////////////////////////////////////


// "key" here starts at 1, "mode" starts at 0

// TODO*** - use 0 based key for consistency

// return current active mode for a key
int livido_key_getmode (gint key);

// returns TRUE if an effect is bound to key/mode
gboolean livido_keymode_valid (gint key, gint mode);

// returns name of effect bound to key/mode
gchar *livido_keymode_get_name (gint key, gint mode);
gchar *livido_filter_get_name(gint filter_idx);


// returns type, "generator", "filter", "transition",...checks either a running instance, or if there is none, the template
gchar *livido_keymode_get_type (gint key, gint mode);

// checks only the template
gchar *livido_filter_get_type(gint idx);

// more general check
gchar *get_type(gint in_channels, gint out_channels);

// returns a filter idx for key/mode
gint livido_keymode_get_idx(gint key, gint mode);

// returns class (instance_template) for a key/mode
const livido_class_t *livido_keymode_get_class(gint key, gint mode);

// returns instance for a key/mode
const livido_instance_t *livido_keymode_get_instance(gint key, gint mode);

// returns plugin name (soname) or empty string
gchar *livido_keymode_get_plugin_name(gint key, gint mode);

// returns FALSE if the keymode is active, or if the new effect is not found: it will match the first name found
gboolean livido_switch_keymode (gint key, gint mode, const gchar *hashname);

// bind an effect to the next free mode on key
// return mode number, or -1 if all slots were full or the effect was not found
int livido_add_effectkey (gint key, gchar *hashname);

// unbind the effect at key/mode: effects at higher numbered modes will slide down to fill the gap
// returns FALSE if not effect was bound there
gboolean livido_delete_effectkey (gint key, gint mode);

// make a hash string of plugin_name, effect_name, author, version - should uniquely identify the effect
gchar *make_livido_hashname(int fx_idx);



//////////////////////////////////////////////////////////////////////////////




//hotkey is 0 based here...
gboolean livido_init_effect(int hotkey, gint rowstride);
void livido_deinit_effect(int hotkey);
void livido_deinit_all(void);
void livido_init_resize(gint rowstride);
void livido_apply_effects(int height, int rowstride, guchar **in_pixel_data, livido_palette_t current_palette, livido_timecode_t tc);

void livido_key_nextmode (void);
gboolean livido_key_valid (int hotkey);
int livido_key_getmode (int hotkey);
int livido_getmodespk (void);

int livido_key_setmode (int hotkey, int mode);


gint livido_get_blend_factor(int hotkey);
void livido_set_blend_factor(int hotkey, gint blend_factor);

// return key number of background generator (or -1)
gint livido_bg_gen_key (void);

// return key number of foreground generator (or -1)
gint livido_fg_gen_key (void);

GdkPixbuf *gdk_pixbuf_new_from_generator (livido_instance_t *src, GError **error);
void livido_swap_fg_bg (void);




